'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var PropTypes = require('prop-types');
var reuni = require('reuni');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = Object.setPrototypeOf ||
    ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
    function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
};

var Provider = (function (_super) {
    __extends(Provider, _super);
    function Provider(props) {
        var _this = _super.call(this, props) || this;
        _this.state = { reuni: reuni.createReuni() };
        return _this;
    }
    Provider.prototype.render = function () {
        return React.Children.only(this.props.children);
    };
    Provider.prototype.getChildContext = function () {
        return {
            reuniInfo: {
                reuni: this.state.reuni,
                isValid: true,
                parentId: null
            }
        };
    };
    Provider.contextTypes = {
        reuniInfo: PropTypes.any
    };
    Provider.childContextTypes = {
        reuniInfo: PropTypes.any
    };
    return Provider;
}(React.Component));

var hasOwn = Object.prototype.hasOwnProperty;
function is(x, y) {
    if (x === y) {
        return x !== 0 || y !== 0 || 1 / x === 1 / y;
    }
    else {
        return x !== x && y !== y;
    }
}
function shallowEqual(objA, objB) {
    if (is(objA, objB))
        return true;
    if (typeof objA !== "object" ||
        objA === null ||
        typeof objB !== "object" ||
        objB === null) {
        return false;
    }
    var keysA = Object.keys(objA);
    var keysB = Object.keys(objB);
    if (keysA.length !== keysB.length)
        return false;
    for (var i = 0; i < keysA.length; i++) {
        if (!hasOwn.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
            return false;
        }
    }
    return true;
}

var Connector = (function (_super) {
    __extends(Connector, _super);
    function Connector(props) {
        var _this = _super.call(this, props) || this;
        _this.mountReuniNode = function (props, reuni$$1, parentId) {
            var thread = props.thread, nodeName = props.nodeName, id = props.id, storeObserver = props.storeObserver, stores = props.stores;
            var node = reuni$$1.mountNode({
                id: id,
                thread: thread,
                name: nodeName,
                parentId: parentId
            });
            _this.setState({ nodeId: node.getId() }, function () {
                stores.forEach(function (_a) {
                    var storeName = _a[0], Store = _a[1], storeOb = _a[2];
                    node.addStore(storeName, Store, storeOb);
                });
                node.observe(storeObserver, function (isValid, entityDict) {
                    if (isValid !== false) {
                        _this.setState({
                            isValid: true,
                            reuni: reuni$$1,
                            entityDict: entityDict
                        });
                    }
                    else {
                        _this.setState({
                            isValid: false
                        });
                    }
                });
            });
        };
        _this.state = {
            childProps: {},
            isValid: false,
            entityDict: null,
            reuni: null,
            nodeId: null
        };
        return _this;
    }
    Connector.prototype.componentWillMount = function () {
        var reuniInfo = this.context.reuniInfo;
        if (reuniInfo != null) {
            var isValid = reuniInfo.isValid, reuni$$1 = reuniInfo.reuni, parentId = reuniInfo.parentId;
            if (isValid !== false) {
                this.mountReuniNode(this.props, reuni$$1, parentId);
            }
        }
        else {
            throw new Error("Node must under Provider Component.");
        }
    };
    Connector.prototype.componentWillReceiveProps = function (nextProps, nextContext) {
        var _this = this;
        var _a = this.props, id = _a.id, stores = _a.stores, thread = _a.thread, storeObserver = _a.storeObserver, nodeName = _a.nodeName, Component = _a.Component;
        if (storeObserver !== nextProps.storeObserver ||
            stores !== nextProps.stores ||
            thread !== nextProps.thread ||
            id !== nextProps.id ||
            nodeName !== nextProps.nodeName) {
            var reuniInfo = nextContext.reuniInfo;
            if (reuniInfo != null) {
                var isValid = reuniInfo.isValid, reuni_1 = reuniInfo.reuni, parentId_1 = reuniInfo.parentId;
                if (isValid !== false) {
                    var nodeId = this.state.nodeId;
                    if (nodeId != null) {
                        reuni_1.unmoutNode(nodeId);
                        this.setState({ isValid: false, nodeId: null }, function () {
                            _this.mountReuniNode(nextProps, reuni_1, parentId_1);
                        });
                    }
                }
            }
            else {
                this.setState({
                    isValid: false
                });
            }
        }
    };
    Connector.prototype.componentWillUnmount = function () {
        var reuniInfo = this.context.reuniInfo;
        var reuni$$1 = reuniInfo.reuni;
        if (this.state.nodeId != null) {
            reuni$$1.unmoutNode(this.state.nodeId);
        }
    };
    Connector.prototype.getChildContext = function () {
        return {
            reuniInfo: {
                isValid: this.state.isValid,
                reuni: this.state.reuni,
                parentId: this.state.nodeId
            }
        };
    };
    Connector.prototype.shouldComponentUpdate = function (nextProps, nextState, nextContext) {
        if (this.state !== nextState) {
            return true;
        }
        if (shallowEqual(this.props, nextProps) !== false) {
            return true;
        }
        if (shallowEqual(this.context, nextContext) !== false) {
            return true;
        }
        if (this.context.router != null &&
            this.context.router.route.location !== nextContext.router.route.location) {
            return true;
        }
        return false;
    };
    Connector.prototype.render = function () {
        var _a = this.state, isValid = _a.isValid, entityDict = _a.entityDict;
        if (isValid !== true)
            return null;
        var _b = this.props, Component = _b.Component, childProps = _b.childProps;
        return React.createElement(Component, __assign({}, Object.assign({}, entityDict, childProps)));
    };
    Connector.contextTypes = {
        reuniInfo: PropTypes.any,
        router: PropTypes.any
    };
    Connector.childContextTypes = {
        reuniInfo: PropTypes.any
    };
    return Connector;
}(React.Component));

var Mount = (function () {
    function Mount(Component, storeObserver, thread) {
        this._Component = Component;
        this._stores = [];
        this._storeObserver = storeObserver;
        this._thread = thread;
    }
    Mount.prototype.setName = function (name) {
        this._name = name;
        return this;
    };
    Mount.prototype.setThread = function (thread) {
        this._thread = thread;
        return this;
    };
    Mount.prototype.addStore = function (storeName, RawStore, storeOb) {
        this._stores.push([storeName, RawStore, storeOb]);
        return this;
    };
    Mount.prototype.toHOC = function () {
        var _this = this;
        return function (props) {
            return React.createElement(Connector, {
                Component: _this._Component,
                childProps: props,
                storeObserver: _this._storeObserver,
                nodeName: _this._name,
                stores: _this._stores,
                thread: _this._thread
            });
        };
    };
    return Mount;
}());
function createMount(thread) {
    if (thread === void 0) { thread = Symbol("thread"); }
    return function (Component, storeObserver) { return new Mount(Component, storeObserver, thread); };
}

exports.storeObserver = reuni.storeObserver;
exports.StoreObserver = reuni.StoreObserver;
exports.value = reuni.value;
exports.task = reuni.task;
exports.store = reuni.store;
exports.delay = reuni.delay;
exports.spawnTask = reuni.spawnTask;
exports.Provider = Provider;
exports.createMount = createMount;
exports.Mount = Mount;
exports.Connector = Connector;
